@@ -26,7 +26,5 @@ class Handler(TCPHandler):
             Point(x=xmax, y=ymin), Point(x=xmax, y=ymax)
         ])
 
-server = LoggingTCPServer(SERVER_ADDR, Handler)
-
 if __name__ == "__main__":
-    server.serve_forever()
+    LoggingTCPServer(SERVER_ADDR, Handler).serve_forever()
