@@ -1,4 +1,4 @@
-# -*- coding: latin-1 -*-
+# -*- coding: utf-8 -*-
 from __future__ import unicode_literals, division
 import os
 import sys
@@ -16,9 +16,10 @@ from unittest import TestCase, main
 import protlib
 from protlib import *
 
-if sys.version_info < (3, 0):
+if sys.version_info[0] == 2:
     from StringIO import StringIO as BytesIO
 else:
+    unicode = str
     from io import BytesIO
 
 warnings.simplefilter("error", CWarning)
@@ -36,9 +37,9 @@ def delete_logs():
         atexit._exithandlers[:] = []    # stop the logging module's exit handler
     
     for suffix in Logger.LEVELS:
-        filename = "unit_tests.{0}_log".format(suffix)
+        filename = Logger.DEFAULT_PREFIX + ".{0}_log".format(suffix)
         if os.path.exists(filename):
-            handlers = logging.getLogger("unit_tests." + suffix).handlers
+            handlers = logging.getLogger(Logger.DEFAULT_PREFIX + "." + suffix).handlers
             if handlers and handlers[0].stream:
                 handlers[0].stream.close()
             os.remove(filename)
@@ -432,7 +433,7 @@ class CTypeTests(TestCase):
         CUnicode(length=5, encoding="utf8", default="hello")
     
     def test_unicode_integrity(self):
-        andre = "andr"
+        andre = b"andr\xc3\xa9".decode('latin-1')
         
         cu = CUnicode(length=6, encoding="latin-1")
         self.assertEqual(andre, cu.parse(cu.serialize(andre)))
@@ -446,7 +447,7 @@ class CTypeTests(TestCase):
         self.assertEqual(p.name, Person.parse(s).name)
     
     def test_unicode_coercion(self):
-        andre = "andr"
+        andre = "andré"
         class Name(CStruct):
             first = CString(length=9)
             last = CUnicode(length=9, encoding="utf8")
@@ -488,7 +489,7 @@ class CTypeTests(TestCase):
         self.assertEqual("\ufffd", p.name)
     
     def test_length_problems(self):
-        self.assertRaises(CWarning, CUnicode(length=5, encoding="utf8").serialize, "andr")
+        self.assertRaises(CWarning, CUnicode(length=5, encoding="utf8").serialize, "andré")
         
         class Person(CStruct):
             name = CString(length=5, default="marvin")
@@ -502,23 +503,23 @@ class CTypeTests(TestCase):
         assert_works_without_warnings(setattr, p, "name", "marvin")
         
         class Person(CStruct):
-            name = CUnicode(length=5, encoding="utf8", default="andr")
+            name = CUnicode(length=5, encoding="utf8", default="andré")
         self.assertRaises(CError, Person)
         assert_works_without_warnings(Person)
         
         class Person(CStruct):
             name = CUnicode(length=5, encoding="utf8")
         p = Person()
-        self.assertRaises(CError, setattr, p, "name", "andr")
-        assert_works_without_warnings(setattr, p, "name", "andr")
+        self.assertRaises(CError, setattr, p, "name", "andré")
+        assert_works_without_warnings(setattr, p, "name", "andré")
         
         class Name(CStruct):
             first = CString(length=5)
             last = CUnicode(length=5, encoding="utf8")
         self.assertRaises(CError, Name, first="marvin")
-        self.assertRaises(CError, Name, last="andr")
+        self.assertRaises(CError, Name, last="andré")
         assert_works_without_warnings(Name, first="marvin")
-        assert_works_without_warnings(Name, last="andr")
+        assert_works_without_warnings(Name, last="andré")
 
 class VarlengthTests(TestCase):
     def test_varlength_structs(self):
@@ -729,7 +730,7 @@ class AutosizedTests(TestCase):
         self.assertEqual(dept, Department.parse(buf))
     
     def test_unicode(self):
-        andre = "andr"
+        andre = "andré"
         
         class Person(CStruct):
             code = CChar(always = 1)
@@ -841,6 +842,7 @@ class BadHandlerTests(TestCase):
         self.assertRaises(CError, EmptyHandler)
     
     def test_duplicate_starts(self):
+        warnings.simplefilter("error", CWarning)
         class DupHandler(ProtHandler):
             class STRUCT_MOD:
                 class Foo(CStruct):
@@ -880,7 +882,7 @@ class LoggerTests(TestCase):
 
 class ServerTestBase:
     def setUp(self):
-        self.reset_logs()        
+        self.reset_logs()
         self.server = self.ServerClass(SERVER_ADDR, self.HandlerClass)
         t = Thread(target=self.server.serve_forever)
         t.daemon = True
@@ -911,12 +913,12 @@ class ServerTestBase:
     
     def reset_logs(self):
         for suffix in Logger.LEVELS:
-            handlers = logging.getLogger("unit_tests." + suffix).handlers
+            handlers = logging.getLogger(Logger.DEFAULT_PREFIX + "." + suffix).handlers
             if handlers and handlers[0].stream:
                 handlers[0].stream.truncate(0)
     
     def read_log(self, name):
-        with open("unit_tests." + name + "_log") as f:
+        with open(Logger.DEFAULT_PREFIX + "." + name + "_log") as f:
             return f.read()
 
 class ServerTests(ServerTestBase):
@@ -1001,7 +1003,7 @@ class TCPTimeoutTests(ServerTestBase, TestCase):
     def test_full_timeout(self):
         before = time()
         self.server.shutdown()
-        self.assertTrue(time() - before > 2)
+        self.assertTrue(time() - before < 3)  # just because we have a 2 second timeout doesn't mean it'll take anywhere close to 2 seconds, but it should always take well under 3
 
 class TCPReadRecvInteract(ServerTestBase, TestCase):
     ServerClass = LoggingTCPServer
